FREE = 0
FIXED = 1

X = 1  # deprecated
Y = 2  # deprecated
ROTZ = 3  # deprecated
DOF_X = 1
DOF_Y = 2
DOF2D_X = 1
DOF2D_Y = 2
DOF2D_ROTZ = 3
DOF2D_PP = 3  # pore pressure
DOF3D_Z = 3
DOF3D_ROTX = 4
DOF3D_ROTY = 5
DOF3D_ROTZ = 6

P = 'P'
M_Z = 'Mz'
V_Y = 'Vy'
M_Y = 'My'
V_Z = 'Vz'
TORSION = 'T'

PLANE_STRAIN = "PlaneStrain"
PLANE_STRESS = "PlaneStress"

# DOFD = {1: ['X'],
#         2: ['X', 'Y'],
#         3: ['X', 'ROTZ']}
